#include "XATICATypes.h"
#ifdef __cplusplus
 extern "C" {
 #endif
  /**
   * Creates a job in the AEM based on the JSDL description. The job can be
   * automatically scheduled or just created, depending on the value of
   * startJob.<br>
   * 
   * If a reservationId is provided, the job will be scheduled on that
   * reservation. Otherwise, a negotiation/reservation process will be started
   * when the job will be scheduled based on resource requirements and
   * scheduling hints. <br>
   * 
   * If calls to VOPS are enabled, policy enforcement gets into the picture.<br>
   * 
   * XOSDCONSOLE annotation denotes that this method becomes a part of XConsole
   * project and therefore be available to user usage and scripts.
   * 
   * @param jsdlFile
   *          is the job description of the job to be created in JSDL format
   *          (content, not path to jsdl file)
   * @param startJob
   *          true if we want to run job now false just to create it without
   *          running
   * @param reservationID
   *          a String with the previously created reservation where the job
   *          will run
   * @param userCtx
   *          the user certificate.
   * @return a String with the JobId of the new job created
   * @throws InvalidCertificateException
   * @throws BadUserCertificateException
   */

int createJob(const char* __jsdlFile,char __startJob,const char* __reservationID,const char* __userCtx, char** returnValue);


  /**
   * Starts the scheduling process of a previously created job. JobId must be a
   * valid jobId in the system.
   * 
   * If a resevationID is provided the job will be scheduled on that
   * reservation. Otherwise, the system will look up the implicit reservation
   * associated to the jobID. System won't select the best reservation.
   * 
   * @param jobId
   *          of the job to be started
   * @param reservationID
   *          the previously created reservation were the job will run
   * @param userCtx
   *          the user certificate.
   * @return an Integer to be deprecated, use Exceptions(XATI)/error
   *         codes(XATICA)
   * @throws InexistentJobIDException
   * @throws InvalidCertificateException
   * @throws OperationNotAllowedException
   * @throws PermissionDeniedException
   * @throws BadUserCertificateException
   */

int runJob(const char* __jobId,const char* __reservationID,const char* __userCtx, int* returnValue);


  /**
   * Starts the scheduling process of a previously created job. JobId must be a
   * valid jobId in the system.
   * 
   * It gets a ResourceID, that should be valid in the reservation provided, and
   * run the job in this resource, if possible.
   * 
   * @param jobId
   *          of the job to be started.
   * @param reservationID
   *          the previously created reservation where the job will be run. If
   *          it's null, the system will first look for the one that was
   *          associated to the job on creation.
   * @param resourceID
   *          of the resource that will run the job (if possible).
   * @param userCtx
   *          the user certificate.
   * @return an Integer to be deprecated
   * @throws ResourceNotInReservationException
   *           if specified resource isn't part of the given reservation.
   */

int runJobRes(const char* __jobId,const char* __reservationID,const CommunicationAddress __resourceID,const char* __userCtx, int* returnValue);


  /**
   * Creates a new Process inside the specified JobID. In order to increase
   * performance, user can specify the ReservationID. Otherwise the system will
   * look for the reservation associated to the job. Finally, a JSDL can be
   * specified to change executable and parameters of the process. <br>
   * Priority for resource selection: <br>
   * Reservation > JSDL > job JSDLInfo <br>
   * 
   * @param jobId
   * @param JSDL
   *          a String with the JSDL content
   * @param reservationId
   * @param resource
   *          : use null, this parameter is deprecated
   * @param userCtx
   * @return an Integer to be deprecated
   * @throws InvalidCertificateException
   *           , BadUserCertificateException
   * @throws PermissionDeniedException
   *           if user is not the owner of the job
   * @throws ResourceNotInReservationException
   *           if resource was provided that is not part of the reservation.
   * @throws OperationNotAllowedException
   *           if resource parameter is used, it's no longer accepted.
   */

int createProcess(const char* __jobId,const char* __JSDL,const char* __reservationId,const CommunicationAddress __resource,const char* __userCtx, int* returnValue);


  /**
   * Apply the operation Control to the specific jobId
   * 
   * @param jobId
   *          of the job to apply the operation
   * @param ctrOp
   *          an Integer defining the operation. Possible values are defined in
   *          enum class
   *          {@link eu.xtreemos.xosd.utilities.ControlOperations#ControlOperations
   *          ControlOperations} E.g. ControlOperations.SUSPENDJOB.val().
   * 
   * @param userCtx
   *          the user certificate
   * @throws InexistentJobIDException
   * @throws InvalidCertificateException
   * @throws OperationNotAllowedException
   * @throws PermissionDeniedException
   */

int jobControl(const char* __jobId,int __ctrOp,const char* __userCtx );


  /**
   * The job identified by the jobId is finished immediately (all the processes
   * of the job) with the exit code provided.
   * 
   * If the process calling exitJob is part of the job this function never
   * returns.
   * 
   * @param jobId
   *          identifying the job to exit
   * @param exitValue
   *          the exit code the job will finish with
   * @param userCtx
   *          the user certificate
   * @throws InexistentJobIDException
   * @throws PermissionDeniedException
   * @throws InvalidCertificateException
   */

int exitJob(const char* __jobId,int __exitValue,const char* __userCtx );


  /**
   * Returns the monitoring information of the requested jobs. Any user of the
   * VO can access monitoring data of the job, but if he is not the owner,
   * access will be restricted to unbuffered system metrics.
   * 
   * @param jobId
   *          an ArrayList of String which is the list of requested jobs.
   * @param flags
   *          an Integer used as a mask to define the metrics to return.
   *          TypeOfInfo enum class has the values of each type. E.g.
   *          TypeOfInfo.BASIC.val(). Several flags might be combined as logical
   *          OR operands. Values and description:
   *          <p>
   *          BASIC: Job identification, status and times
   *          <p>
   *          JOB_DEFINITION: The JSDL
   *          <p>
   *          RESOURCES_ALLOCATED: ReservationIDs
   *          <p>
   *          RESOURCES_CONSUMED: Extended information about resources (to be
   *          implemented)
   *          <p>
   *          USER_METRICS: Metrics added by the user with the addJobMetric
   *          method
   * @param infoLevel
   *          an Integer defining the sources of information allowed in the
   *          operation. InfoLevel enum class should be used to get the right
   *          values. E.g. InfoLevel.PROCESS.val(). Values and description:
   *          <p>
   *          JOB: it communicates only with the jobMng.
   *          <p>
   *          PROCESS: it communicates both with jobMng and exec- or resMng.
   *          <p>
   *          KERNEL: it communicates with every needed service and also
   *          considers special kernel metrics (to be implemented)
   * @param metrics
   *          a list of metrics to obtain, additional to the flags. If any of
   *          the metrics doesn't exist, it throws a MonitorException.
   * @param userCtx
   *          the X509Certificate Object with the user certificate. If user
   *          doesn't have permission for any of the jobs, it throws a
   *          PermissionDeniedException.
   * @return XML String containing the information of the Jobs. If any of the
   *         jobs doesn't exist, it simply gets discarded.
   * @throws InvalidCertificateException
   *           , PermissionDeniedException,
   * @throws MonitorException
   *           on bad list of metrics.
   * @throws OperationNotAllowedException
   *           on wrong flags/infoLevel.
   * @throws Exception
   */

int getJobsInfo(const ArrayList __jobIds,int __flags,int __infoLevel,const ArrayList __metrics,const char* __userCtx, char** returnValue);


  /**
   * TODO: To be eventually deprecated. It's not in the monitoring deliverable
   * 
   * @see #getJobsInfo(ArrayList, Integer, Integer, ArrayList, X509Certificate)
   */

int getJobInfo(const char* __jobId,int __flags,int __infoLevel,const ArrayList __metricsList,const char* __userCtx, char** returnValue);


  /**
   * Returns the jobID corresponding to a <resourceId, pid>
   * 
   * @param resourceId
   *          the CommunicationAddress of the resource where the process is
   *          assigned
   * @param pid
   *          an Integer with the process identifier local to that resource
   * @param userCtx
   *          an X509Certificate object with the user certificate
   * @return a String representing the jobId
   * @throws InvalidCertificateException
   * @throws Exception
   *           when no jobID associated with the <resource, pid>
   * @throws PermissionDeniedException
   *           when user is no the owner of the job
   */

int getJobId(const CommunicationAddress __resourceId,int __pid,const char* __userCtx, char** returnValue);


  /**
   * Returns all the jobIDs of the jobs belonging to the given user
   * 
   * @param userId
   *          a String identifying the user we want to obtain the jobs from, by
   *          its Distinguished Name. If not specified, it uses the one in the
   *          certificate.
   * @param userCtx
   *          a X509Certificate object with the user certificate
   * @return an ArrayList of Strings representing the jobIDs
   * @throws InvalidCertificateException
   * @throws BadUserCertificateException
   */

int getJobsUser(const char* __userId,const char* __userCtx, ArrayList* returnValue);


  /**
   * Sends an event to a job
   * 
   * @param signal
   *          an Integer with the signal number to be sent
   * @param operation
   *          can be: 0: ALL every jobUnit of the job 1: MASTER undefined 2:
   *          SPECIFIED every jobUnit specified in the list 3: EXCEPT every
   *          jobUnit with the exception of the specified in the list
   * @param jobId
   *          a String identifying the job we are going to send the signal to
   * @param list
   *          an ArrayList of CommunicationAddress
   * @param userCtx
   *          the user certificate
   * 
   * @throws InexistentJobIDException
   *           , InvalidCertificateException
   */

int sendEvent(const char* __jobId,int __signal,int __operation,const ArrayListCA __list,const char* __userCtx );


  /**
   * Blocks the calling process until the job indicated finishes
   * 
   * @param jobId
   *          identifying the job the calling process is waiting for
   * @return the exit value of the job
   * @throws InvalidCertificateException
   *           , PermissionDeniedException
   */

int jobWait(const char* __jobId,const char* __userCtx, int* returnValue);


  /**
   * This function adds a new job dependence. Dependence will be bidirectional:
   * <ul>
   * fromJobId ---DOWN--> toJobId
   * </ul>
   * <ul>
   * fromJobId <-- UP --- toJobId
   * </ul>
   * Take into account that inverting parameters makes a difference because of
   * the different up/down lists. Unidirectional dependences might be added with
   * other methods specified below<br>
   * 
   * We support sets of dependencies identified by TAG, to be able to use them
   * for different purposes. AEM won't interpret these TAGS, just group
   * dependencies of jobs based on them. AEM won't check cycles in job
   * dependencies. It is user/job responsibility.
   * 
   * @see #addDependenceUp(String, String, String, X509Certificate)
   * @see #addDependenceDown(String, String, String, X509Certificate)
   * @param fromJobId
   *          the job from which tojobId depends
   * @param toJobId
   *          the job that depends on fromJobId
   * @param TAG
   *          is a name to identify a set of dependencies.
   * @param userCtx
   *          the user certificate, only the owner of the job can manage
   *          dependences.
   * @throws InexistentJobIDException
   *           if jobId doesn't exist in the system
   * @throws PermissionDeniedException
   *           if user is not the owner of the job
   * @throws BadUserCertificateException
   *           if certificate is incorrect
   */

int addDependence(const char* __fromJobId,const char* __toJobId,const char* __TAG,const char* __userCtx );


  /**
   * Deletes an existing bidirectional dependence between two jobs.<br>
   * Take into account that inverting parameters makes a difference because of
   * the different up/down lists.
   * 
   * @see #addDependence(String, String, String, X509Certificate)
   * @param FromJobId
   *          the job from which tojobId depends
   * @param toJobId
   *          the job that depends on fromJobId
   * @param TAG
   *          is a name to identify a set of dependencies.
   * @param userCtx
   *          the user certificate, only the owner of the job can manage
   *          dependences.
   * @throws InexistentJobIDException
   *           if jobId doesn't exist in the system
   * @throws PermissionDeniedException
   *           if user is not the owner of the job
   * @throws BadUserCertificateException
   *           if certificate is incorrect
   */

int deleteDependence(const char* __FromJobId,const char* __ToJobId,const char* __TAG,const char* __userCtx );


  /**
   * Returns the list of jobs that have a dependence to the given job. Take into
   * account, dependences are directional, and named. This method is recursive
   * and user may specify a depth limit. On no limit searches, an alternative
   * algorithm that detects cycles is applied.
   * 
   * @param jobID
   *          a String with the identifier of the starting job to get the
   *          related jobs
   * @param tag
   *          a String with the name to identify a set of dependencies.
   * @param levels
   *          an Integer with the recursion depth. For no limit, use 0 or
   *          negative.
   * @param direction
   *          an Integer to select the type of dependence, as defined in enum
   *          class
   *          {@link eu.xtreemos.xosd.utilities.dependences.DependenceDirection#DependenceDirection
   *          DependenceDirection} E.g. DependenceDirection.UP.val()
   * @param userCtx
   *          the user certificate, only the owner of the job can manage
   *          dependences.
   * @return list of jobIds that have the specified (UP or DOWN) dependence to
   *         the given job and so recursively for the given levels. No
   *         particular order is guaranteed.
   * @throws InexistentJobIDException
   * @throws OperationNotAllowedException
   * @throws PermissionDeniedException
   *           if user is not the owner of the job
   * @throws BadUserCertificateException
   *           if certificate is incorrect
   */

int getListOfDependences(const char* __jobID,const char* __tag,int __levels,int __direction,const char* __userCtx, ArrayList* returnValue);


  /**
   * Modifies the hints passed in createJob function. Only Scheduling hints
   * implemented. Use before the job is Running.
   * 
   * @param jobId
   *          identifying the job to change the requirements of
   * @param operation
   *          an Integer (or use JobHints.val()) that can be:
   *          <ul>
   *          0: MIGRATION_REQ
   *          </ul>
   *          <ul>
   *          1: SCHEDULING_REQ
   *          </ul>
   * @param requirements
   *          an Integer (or use SchedulingHints.val()) that can be:
   *          <ul>
   *          SCHEDULING_REQ:
   *          </ul>
   *          <ul>
   *          0: DEFAULT Use normal scheduler decision
   *          </ul>
   *          <ul>
   *          1: ONEPERNODE Try to create one process (CreateProcess) on each
   *          resource. (MPI)
   *          </ul>
   * @param wait
   *          if the call is synchronous or asynchronous (not implemented)
   * @param userCtx
   *          the user certificate
   * @throws InvalidCertificateException
   *           , PermissionDeniedException, InexistentJobIDException,
   *           OperationNotAllowedException
   */

int updateJobHints(const char* __jobId,int __operation,int __requirements,char __wait,const char* __userCtx );


  /**
   * Returns the list of available metrics for a specific job, both system and
   * user defined.
   * 
   * @param jobId
   *          a String identifying the job to get the metrics from, if the jobId
   *          is null the generic list of metrics is returned
   * @param userCtx
   *          a X509Certificate object with the user certificate
   * @return an ArrayList of MetricsDesc objects corresponding to the metrics
   *         associated to the job. If the user is not the owner of the job a
   *         generic list of metrics is also returned. The ones he'll be able to
   *         get indeed.
   * @throws InvalidCertificateException
   * @throws PermissionDeniedException
   *           if user is not in the VO owner of the job.
   * @throws InexistentJobIDException
   */

int getJobMetrics(const char* __jobId,const char* __userCtx, ArrayList* returnValue);


  /**
   * Sets the value of a Metric. A metric will be an user-defined attribute of
   * the job. Not all the attributes can be set, for instance the user time or
   * the status are set by the system, not by the user.
   * 
   * @param jobId
   *          a String identifying the job, the metric is associated to.
   * @param metricName
   *          is a String with the name of the metric to be set.
   * @param value
   *          a String with the value to be set to the metric. It's a string,
   *          but it will be parsed according to the metric type. If it's time
   *          type, the string must be in jiffies format.
   * @return an Integer to be deprecated.
   * @throws InvalidCertificateException
   * @throws InexistentJobIDException
   * @throws PermissionDeniedException
   * @throws OperationNotAllowedException
   * @throws BadMetricException
   */

int setMetricValue(const char* __jobId,const char* __metricName,const CommunicationAddress __resourceID,int __pid,const char* __value,const char* __userCtx, int* returnValue);


  /**
   * Switches on and off buffering for the specified metric. With buffering on,
   * multiple values of a metric are stored, and its timestamps represent the
   * time when the value changed. Only metrics defined as "bufferable" on
   * creation can be buffered.
   * 
   * @param jobId
   *          a String identifying the job whose metric is about to get
   *          buffered. Its worth noticing that buffering is not applied to the
   *          whole job but to the smallest part of it according to the scope.
   * @param metricName
   *          a String with the name of the metric.
   * @param resourceID
   *          the CommunicationAddress of the resource where the buffering is to
   *          be applied. Used only if the scope of the metric is resource,
   *          jobUnit or process.
   * @param pid
   *          Integer with the process ID of the process where the buffering is
   *          to be applied. Used only if the scope of the metric is process.
   * @param flags
   *          an Integer mask with options to aggregate several metrics, tag
   *          buffers and enable or disable buffering.
   *          <ul>
   *          Use flags |= TypeOfInfo.ENABLE.val() to enable buffering
   *          </ul>
   *          <ul>
   *          flags |= ~TypeOfInfo.ENABLE.val() to disable buffering
   *          </ul>
   *          TODO: add support for metric aggregations, if required.
   * @param userCtx
   *          a X509Certificate object with the user certificate.
   * @return an Integer to be deprecated
   * @throws InexistentJobIDException
   * @throws OperationNotAllowedException
   * @throws PermissionDeniedException
   */

int setMonitorBuffering(const char* __jobId,const char* __metricName,const CommunicationAddress __resourceID,int __pid,int __flags,const char* __userCtx, int* returnValue);


  /**
   * Adds a new user defined metric to the job. Afterwards, user might give
   * values to it through the setMetricValue service method and get them with
   * getJobsInfo. Metrics are checked for correctness before insertion. Service
   * might communicate with other services and XOSDs if needed.
   * 
   * @param jobId
   *          a String identifying the job to add the metric to
   * @param metric
   *          a MetricsDesc object with the metric to be added
   * @param userCtx
   *          a X509Certificate object with the user certificate
   * @return an Integer to be deprecated.
   * @throws OperationNotAllowedException
   *           if a metric with the same name already exists.
   * @throws PermissionDeniedException
   *           if the user is not the owner of the job.
   * @throws BadMetricException
   *           if the metric description doesn't pass the correctness tests.
   */

int addJobMetric(const char* __jobId,const MetricsDesc __metric,const char* __userCtx, int* returnValue);


  /**
   * Not defined yet
   */

int updateJobMetric(const char* __jobId,const MetricsDesc __metric,const char* __userCtx );


  /**
   * Removes a user defined metric from the job. Data associated to the metric
   * is also erased. It might communicate with other services and XOSDs if
   * needed to propagate the effect.
   * 
   * @param jobId
   *          a String identifying the job to remove the metric from
   * @param metricName
   *          a String with the name of the metric to be removed
   * @param userCtx
   *          a X509Certificate object with the user certificate
   * @return an Integer to be deprecated.
   * @throws InvalidCertificateException
   * @throws OperationNotAllowedException
   *           if metric is system type or doesn't exist
   * @throws PermissionDeniedException
   */

int removeJobMetric(const char* __jobId,const char* __metricName,const char* __userCtx, int* returnValue);


  /**
   * identify job manager address of each job
   */

int identifyJobManagers(const ArrayList __dependentJobs,const char* __initialJobId,const char* __strategy,const char* __options,const char* __mode,const char* __cert );


  /**
   * lock a checkpoint action - private member jobslist needed piggyback jsdl
   * file and executable name
   */

int lockCheckpoint(const char* __jobId,const char* __initialJobId,const char* __userCert,const CommunicationAddress __jobCpAddr,const CommunicationAddress __superJobCpAddr,const char* __strategy,const char* __options,const char* __mode,const ArrayList __dependentJobs );




int unlockCheckpoint(const char* __jobId );




int unlockRestart(const char* __jobId );


  /**
   * set up structures that represent a job in AEM at restart Eugen has added
   * code for recreating the job correctly at restart time
   */

int recreateJob(const char* __jobId,const char* __initialJobId,const char* __jsdlFileContent,const char* __checkpointVersion,const CommunicationAddress __superJobCpAddr,const ArrayListCA __jobResourceList,const char* __userCert );


  /**
   * Changes the UID associated to a jobId
   * 
   * @param jobId
   *          identifying the job whose user will change
   * @param oldUserCtx
   *          the certificate of the current user
   * @param newUserCtx
   *          the new certificate to be used
   */

int chgUID(const char* __jobId,const char* __oldUserCtx,const char* __newUserCtx );


  /**
   * Modifies the requirements passed in createJob function.
   * 
   * @param jobId
   *          identifying the job to change the requirements of
   * @param operation
   *          can be: 0: ADD_RESOURCES 1: RELEASE_RESOURCES 2: EXTEND_RESOURCES
   * 
   * @param requirements
   *          in a JSDL format specifying the new requirements
   * @param wait
   *          if the call is synchronous or asynchronous
   * @param userCtx
   *          the user certificate
   */

int updateJobRequirements(const char* __jobId,int __operation,const char* __requirements,char __wait,const char* __userCtx );


  /**
   * Update the characteristics of the monitorization of a job. It allows to
   * start We can use start,stop to enable/disable all the callbacks with one
   * XOS call. It is still pending to decide if we are going to use start/stop
   * to automatically call external start/stop functions
   * 
   * @param jobId
   * @param op
   *          can be: 0:START 1:STOP 2:CHANGE_LEVEL
   * @param level
   *          the new monitoring level in case op is set to CHANGE. Can be:
   *          High, medium or Low
   * @param userCtx
   *          the user certificate
   */

int jobMonitoringControl(const char* __jobId,int __op,const char* __level,const char* __userCtx );


  /**
   * Blocks the calling process until "event" is received.
   * 
   * @param jobId
   * @param event
   *          the event the process is waiting for
   * @param userCtx
   *          the user certificate
   * @return
   */

int waitForEvent(const char* __jobId,int __event,const char* __userCtx );


  /**
   * Adds a monitoring callback which is used for notifying on metric value
   * changes. Don't use this one from XATICA, use addMonitoringCallbackMD
   * instead.
   * 
   * @param jobId
   *          Id of the monitored job.
   * @param metricEvent
   *          Description of the event to monitor.
   * @param callback
   *          callback to be added.
   */

int addMonitoringCallback(const char* __jobId,const MetricEvent __metricEvent,const char* __callback,const char* __userCtx );


  /**
   * Adds a monitoring callback which is used for notifying on metric value
   * changes. This is the version to use in Mobile devices. No callback is
   * specified. The invocation returns on event. Errors should be handled as
   * usual in XATI/XATICA.
   * 
   * @param jobId
   *          Id of the monitored job.
   * @param metricEvent
   *          Description of the event to monitor.
   * @return Integer: undefined. Event happened.
   */

int addMonitoringCallbackMD(const char* __jobId,const MetricEvent __metricEvent,const char* __userCtx, int* returnValue);




int killAllByUser(const char* __userID,const char* __credential );




int getVersion( int* returnValue);


#ifdef __cplusplus 
 } 
 #endif 
