#include "XATICATypes.h"
#ifdef __cplusplus
 extern "C" {
 #endif
	/**
	 * Creates an empty reservation and returns its reservation ID.
	 * 
	 * @param userCertificate
	 *            The certificate of the user that will be the owner of the
	 *            reservation.
	 * @return the new reservation ID.
	 * @throws BadUserCertificateException
	 *             if the certificate provided is missing any needed attributes.
	 */

int createEmptyReservation(const char* __userCertificate, char** returnValue);


	/**
	 * Creates a reservation based on the JSDL query.
	 * 
	 * @param query
	 *            The content of the JSDL document containing the job
	 *            description and resource requirements.
	 * @param userCertificate
	 *            The user that is making the reservation.
	 * @return The ID of the newly created reservation.
	 * @throws Exception
	 *             if the resource selection fails, if the resource selection
	 *             yields a lower number of candidates than requested in the
	 *             JSDL, if the selected nodes have no free slots
	 * 
	 */

int createReservation(const char* __query,const char* __userCertificate, char** returnValue);


	/**
	 * Creates a reservation based on the list of local allocation requests. If
	 * any of the requests fail, the whole reservation fails.
	 * 
	 * @param requests
	 *            The collection of reservation requests directed at individual
	 *            local requests.
	 * @param userCertificate
	 *            The certificate of the user creating the reservation.
	 * @return The ID of the newly created reservation, or null if the
	 *         reservation has failed.
	 * @throws Exception
	 */

int createReservationExplicit(const LinkedList __requests,const char* __userCertificate, char** returnValue);


	/**
	 * Updates the reservation by adding the local reservations as discovered
	 * according to the JSDL query. The reservation that is to be updated has to
	 * be empty.
	 * 
	 * @param reservationId
	 *            The ID of the reservation to update.
	 * @param query
	 *            The JSDL query.
	 * @param userCertificate
	 *            The certificate of the user performing the action.
	 * @return True if the reservation succeeds.
	 * @throws Exception
	 *             If the reservation doesn't exist, if the user cannot update
	 *             the reservation (ownership), or if the reservation is not
	 *             empty, if the resource selection fails, if the resource
	 *             selection yields a lower number of candidates than requested
	 *             in the JSDL, if the selected nodes have no free slots.
	 */

int updateReservation(const char* __reservationId,const char* __query,const char* __userCertificate, char* returnValue);


	/**
	 * Update previously created reservations.
	 * 
	 * @param reservationId
	 * @param requests
	 * @param userCertificate
	 * @return
	 * @throws Exception
	 */

int updateReservationExplicit(const char* __reservationId,const LinkedList __requests,const char* __userCertificate, char* returnValue);


	/**
	 * Drop the reservation. The method releases all the local allocations, then
	 * removes the reservation from the directory. If any of the allocations
	 * cannot be released, it keeps them in the directory, as well as the
	 * reservation Id, so that they can be released later.
	 * 
	 * @param reservationId
	 *            The ID of the reservation to be released.
	 * @param userCertificate
	 *            The certificate of the user performing the action. If the
	 *            parameter is null, the method will assert that the call came
	 *            from a trusted service.
	 * @return True if the reservation has been fully released and the
	 *         reservation ID removed from the directory. False if any of the
	 *         local allocations could not be released.
	 * @throws Exception
	 */

int releaseReservation(const char* __reservationId,const char* __userCertificate, char* returnValue);


	/**
	 * Attach a job to the reservation.
	 * 
	 * @param reservationId
	 *            The ID of the reservation to attach the job to.
	 * @param jobId
	 *            The ID of the job to be attached to the job.
	 * @param userCertificate
	 *            The certificate of the user attempting the operation.
	 * @return
	 * @throws Exception
	 */

int attachJob(const char* __reservationId,const char* __jobId,const char* __userCertificate, int* returnValue);


	/**
	 * Detach the job from the reservation.
	 * 
	 * @param reservationId
	 *            The ID of the reservation the job is to be detached from.
	 * @param jobId
	 *            The ID of the job to be detached from the reservation.
	 * @param userCertificate
	 *            The certificate of the user attempting the operation.
	 * @return
	 * @throws Exception
	 */

int detachJob(const char* __reservationId,const char* __jobId,const char* __userCertificate, int* returnValue);


	/**
	 * Query the reservation ID that a job is attached to.
	 * 
	 * @param jobId
	 *            The job ID we are querying the reservation of.
	 * @param userCertificate
	 *            The certificate of the user attempting the action.
	 * @return
	 * @throws Exception
	 *             if the user does not have access to the reservation.
	 */

int getReservationFromJob(const char* __jobId,const char* __userCertificate, char** returnValue);


	/**
	 * Get the list of the resources and the related time constraints related to
	 * the reservation.
	 * 
	 * @param reservationId
	 *            The ID of the reservation we are querying.
	 * @param userCertificate
	 *            The certificate of the user attempting the operation.
	 * @return
	 * @throws Exception
	 */

int getReservationResources(const char* __reservationId,const char* __userCertificate, LinkedList* returnValue);


	/**
	 * Returns all free slots in a timetable for some metric on the node. The
	 * call performs the access rights check.
	 * 
	 * @param resourceID
	 * @return null if no such resource exists or there was error with attribute
	 *         conformance.
	 */

int getAllFreeSlotsFor(const CommunicationAddress __nodeAddress,const char* __resourceID,const char* __userCert, FreeSlots* returnValue);


	/**
	 * Returns all free slots for a given resource and time frame on the node.
	 * The call performs the access rights check.
	 * 
	 * @param resourceID
	 * @param from
	 * @param to
	 * @return
	 */

int getFreeSlotsFor(const CommunicationAddress __nodeAddress,const char* __resourceID,const GregorianCalendar __from,const GregorianCalendar __to,const char* __userCert, FreeSlots* returnValue);


	/**
	 * Retrieve the list of reservation IDs owned by the user.
	 * 
	 * @param userCertificate
	 *            The certificate of the user.
	 * @return A collection of the IDs of the reservations owned by the user.
	 * @throws Exception
	 */

int getUserReservations(const char* __userCertificate, ArrayList* returnValue);


#ifdef __cplusplus 
 } 
 #endif 
